import numpy as np
import matplotlib.pyplot as plt

def mostrar_datos(features, labels, features_names = ['Feature 1', 'Feature 2']):

    plt.figure(figsize=(5, 5), dpi=100)
    clases = np.unique(labels).astype(int)
    for c in clases:
        plt.scatter(features[labels==c,0], features[labels==c,1], label=f'Clase {c}')

    plt.xlabel(features_names[0])
    plt.ylabel(features_names[1])
    
    plt.title('Datos')

    plt.legend()
    
    plt.grid()
    plt.show()

def mostrar_superficie_decision(features, labels, clf_svm, clf_logreg, objects, op_type1_1, op_type2_1):
    ''' 
    función auxiliar para mostrar el resultado de la clasificación 
    '''
    
    plt.figure(figsize=(5, 5), dpi=100)
    clases = np.unique(labels).astype(int)
    for c in clases:
        indices = labels==c
        plt.scatter(features[indices,0], features[indices,1], label=objects[c])

    # plot the decision svm
    ax = plt.gca()
    xlim = ax.get_xlim()
    ylim = ax.get_ylim()
    # create grid to evaluate model
    xx = np.linspace(xlim[0], xlim[1], 30)
    yy = np.linspace(ylim[0], ylim[1], 30)
    YY, XX = np.meshgrid(yy, xx)
    xy = np.vstack([XX.ravel(), YY.ravel()]).T
    Z = clf_svm.decision_function(xy).reshape(XX.shape)
    # plot decision boundary and margins
    ax.contour(XX, YY, Z, colors='k', levels=[-1, 0, 1], alpha=0.5,
               linestyles=['--', '-', '--'])
    # plot support vectors
    ax.scatter(clf_svm.support_vectors_[:, 0], clf_svm.support_vectors_[:, 1], s=100,
               linewidth=1, facecolors='none', edgecolors='k', label='vectores de soporte')

    # plot regresión logística
    w_log = clf_logreg.coef_[0]
    b_log = clf_logreg.intercept_[0]
    #ap = -w_log[0] / w_log[1]
    #ymin = features[:, 1].min()
    #ymax = features[:, 1].max()
    #if ap >= 0:
    #    xmin = np.max([(ymin + b_log / w_log[1]) / ap, features[:, 0].min()])
    #    xmax = np.min([(ymax + b_log / w_log[1]) / ap, features[:, 0].max()])
    #else:
    #    xmin = np.max([(ymax + b_log / w_log[1]) / ap, features[:, 0].min()])
    #    xmax = np.min([(ymin + b_log / w_log[1]) / ap, features[:, 0].max()])
    xmin = features[:, 0].min()
    xmax = features[:, 0].max()
    xx = np.linspace(xmin, xmax)
    yy = -( w_log[0] * xx + b_log) / w_log[1]
    plt.plot(xx, yy, 'r-', label='límite de decisión de regresión logística')
    
    plt.legend()
    plt.xlabel('ZCR ' + op_type1_1)  
    plt.ylabel('RMS ' + op_type2_1)  

    plt.title('Superficie de decisión')
    
    plt.grid()
    plt.show()
    