import re
import string
from nltk.stem import PorterStemmer  # Revisar si esta librería está disponible .....

def preprocesar_correo(email_contents):
    """
    Procesa el contenido de un correo electrónico aplicando una serie de transformaciones para
    limpiarlo y preparar sus características para un modelo de aprendizaje automático.
    
    Args:
        email_contents (str): Texto del correo electrónico a procesar.

    Returns:
        str: Texto procesado y transformado, listo para usar en un modelo.
    """

    # Convertir a minúsculas para estandarizar el texto
    processed = email_contents.lower()

    # Eliminar etiquetas HTML
    processed = re.sub('<[^<>]+>', ' ', processed)

    # Reemplazar números con el texto 'number'
    processed = re.sub('[0-9]+', 'number', processed)

    # Sustituir URLs con 'httpaddr'
    processed = re.sub('(http|https)://[^\s]*', 'httpaddr', processed)

    # Reemplazar direcciones de correo electrónico con 'emailaddr'
    processed = re.sub('[^\s]+@[^\s]+', 'emailaddr', processed)

    # Cambiar el símbolo de dólar '$' por 'dollar'
    processed = re.sub('[$]+', 'dollar', processed)
    
    # Eliminar caracteres de puntuación, reemplazándolos con espacios
    for punctuation in string.punctuation:
        processed = processed.replace(punctuation, ' ')
        
    # Instanciar el stemmer de Porter
    stemmer = PorterStemmer()
    
    # Aplicar stemmización y limpieza de caracteres no alfanuméricos palabra por palabra
    processed = ' '.join([stemmer.stem(re.sub('[^a-zA-Z0-9]', '', word)) for word in processed.split()])
    
    # Eliminar espacios redundantes
    processed = ' '.join(processed.split())  
    
    # Retornar el texto procesado y limpiado
    return processed.strip()

