#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon May 13 07:50:55 2019

@author: fuaa
"""
from matplotlib import pyplot as plt
import numpy as np
import getpass
import socket
from matplotlib.colors import LinearSegmentedColormap
from utils.ejercicio_kmeans_docentes import *

_THRESHOLD = 10e-9


#########################################################################
def validar_resultado(*args, **kwargs):
    """"
    Función para validar resultado a invocar desde el notebook.
    """
    # _ES_CORRECTO = False
    _DEBUG = False

    # Abrir el cartel del validación.
    print("+-------------------------------------------------------------------------+")
    print("|               FuAA (2do. parcial 2024): validar resultado               |")
    print("+-------------------------------------------------------------------------+")
    for key, value in kwargs.items():
        if key == "debug":
            _DEBUG = value
    if _DEBUG:
        print('args:', args)
        print('kwargs:', kwargs)

    if (len(args) == 0):
        print("| Sin opciones para evaluar.                                              |")
        print("+-------------------------------------------------------------------------+")
        return False

    ###########################################################
    # Ejercicio 1. K-Means 
    ###########################################################
    elif (args[0] == "k_means"):
        if 'funcion' in kwargs:
            test_algoritmo = kwargs['funcion']
        else:
            print( "[validar_resultado] Error: llamada sin argumentos (%s)" % args[0] )
            return
        
        X = np.ones((10,4))
        X[:,1] = np.arange(10)
        X[:,2] = np.linspace(-5,5,10)
        X[:,3] = np.linspace(-4,-100,10)
        
        centros_true = np.array([[1.00000000e+00, 8.00000000e+00, 3.88888889e+00, -8.93333333e+01],
                                [1.00000000e+00, 1.00000000e+00, -3.88888889e+00, -1.46666667e+01],
                                [1.00000000e+00, 4.50000000e+00, 1.11022302e-16, -5.20000000e+01]])
        etiquetas_true = np.array([1, 1, 1, 2, 2, 2, 2, 0, 0, 0])
       
        centros, etiquetas = test_algoritmo(X=X, k=3, semilla=42)
        
        # Validar dimensión de datos transformados.
        condicion1 = centros.shape == centros_true.shape
        fuaa_assert(condicion1,
                    mensajeFalse="La dimensión de los centros no es correcta.",
                    mensajeTrue="Dimensión de centros: resultado validado.")

        # Validar transformación.
        condicion2 = son_iguales(centros_true, centros)
        fuaa_assert(condicion2,
                    mensajeFalse="El resultado de los centros no es válido.",
                    mensajeTrue="Centros: resultado validado.")
        
        # Validar dimensión de datos transformados.
        condicion3 = etiquetas.shape == etiquetas_true.shape
        fuaa_assert(condicion3,
                    mensajeFalse="La dimensión de las etiquetas no es correcta.",
                    mensajeTrue="Dimensión de las etiquetas: resultado validado.")

        # Validar transformación.
        condicion4 = son_iguales(etiquetas_true, etiquetas)
        fuaa_assert(condicion4,
                    mensajeFalse="El resultado de las etiquetas no es válido.",
                    mensajeTrue="Etiquetas: resultado validado.")
    ###########################################################
    # Ejercicio 1. Propagación de etiquetas aleatorias 
    ###########################################################
    elif (args[0] == "propagar_primeras_50_etiquetas"):
        if 'funcion' in kwargs:
            test_algoritmo = kwargs['funcion']
        else:
            print( "[validar_resultado] Error: llamada sin argumentos (%s)" % args[0] )
            return
        
        np.random.seed(33)
        X = np.random.randint(0, 256, (200, 64))
        y = np.random.randint(0, 10, 200)
        y_labeled = y[:50]

        y_true = propagar_primeras_50_etiquetas_fuaa(X, y_labeled)

        y_prop = test_algoritmo(X_train=X, y_train=y_labeled)
       
        # Validar dimensión de datos transformados.
        condicion1 = len(y_true) == len(y_prop)
        fuaa_assert(condicion1,
                    mensajeFalse="La dimensión del vector de etiquetas no es correcta.",
                    mensajeTrue="La dimensión del vector de etiquetas: resultado validado.")

        # Validar transformación.
        condicion2 = son_iguales(y_prop, y_true)
        fuaa_assert(condicion2,
                    mensajeFalse="El resultado de las etiquetas no es válido.",
                    mensajeTrue="Etiquetas: resultado validado.")
    
    ###########################################################
    # Ejercicio 1. Propagación de etiquetas K-Means
    ###########################################################
    elif (args[0] == "propagar_etiquetas_k_means"):
        if 'funcion' in kwargs:
            test_algoritmo = kwargs['funcion']
        else:
            print( "[validar_resultado] Error: llamada sin argumentos (%s)" % args[0] )
            return
        
        np.random.seed(33)  
        asignaciones = [ 4, 31,  9, 33, 43, 34, 47, 48, 49, 49, 29,  0,  7, 45, 42, 26, 30,
       43, 45, 48, 13, 20,  0, 13, 29, 44, 31, 36, 38, 35, 46, 44,  8, 45,
       11, 10,  7,  2, 21, 37,  1, 48, 26, 44, 44,  7, 24, 37, 45,  0, 22,
       36, 12, 48, 12, 27, 43,  5, 21, 18, 35, 39, 14, 45, 14, 48, 16, 48,
       46, 34, 19, 11, 44, 15, 29, 18,  1, 48, 27, 49, 32, 27, 35,  2, 48,
       23, 34,  6, 15, 30, 22, 27, 24,  5,  3, 41, 31, 23, 35, 27, 48,  1,
       26, 13, 13, 15, 42, 38, 30,  5,  0, 46, 30, 32,  0,  3, 30, 29, 25,
       14, 41, 34, 46, 32,  8, 19, 19, 36, 23,  1, 44, 16, 19,  7, 26, 42,
       34, 13, 37, 14, 48, 26,  0, 32,  7,  2, 46, 45, 18,  8, 29, 12,  1,
        5, 35, 47, 30, 15, 18, 45, 48,  6, 23, 35,  4, 27, 18, 19, 18, 41,
        6, 24,  1,  0,  6, 21, 14, 15, 18, 27, 35, 35, 47, 29, 16, 37, 38,
        0,  6, 35, 11, 11,  5,  5, 18, 15,  5,  3, 10, 47]

        etiquetas_representativas = [6, 9, 8, 4, 2, 7, 3, 2, 0, 9, 5, 4, 9, 4, 9, 6, 1, 3, 7, 1, 7, 9,
       4, 9, 4, 3, 0, 7, 3, 3, 6, 0, 3, 4, 5, 1, 3, 6, 0, 5, 9, 0, 1, 1,
       2, 5, 2, 5, 8, 4]
        
        y_true = propagar_etiquetas_k_means_fuaa(asignaciones, etiquetas_representativas)
        y_prop = test_algoritmo(asignaciones=asignaciones, etiquetas_representativas=etiquetas_representativas)
       
        # Validar dimensión de datos transformados.
        condicion1 = len(y_true) == len(y_prop)
        fuaa_assert(condicion1,
                    mensajeFalse="La dimensión del vector de etiquetas no es correcta.",
                    mensajeTrue="La dimensión del vector de etiquetas: resultado validado.")

        # Validar transformación.
        condicion2 = y_prop == y_true#son_iguales(y_prop, y_true)
        fuaa_assert(condicion2,
                    mensajeFalse="El resultado de las etiquetas no es válido.",
                    mensajeTrue="Etiquetas: resultado validado.")
    
 
#########################################################################
def identificar_parcial():
    username = getpass.getuser()
    hostname = socket.gethostname()
    print("Usuario %s en %s." % (username, hostname))


#########################################################################
def printcolor(mensaje, color="k"):
    """
    Función para imprimir en colores y con el formato de interés.
    """
    if (color == "r"):
        mensajeColor = "\x1b[31m" + mensaje + "\x1b[0m"
    elif (color == "g"):
        mensajeColor = "\x1b[32m" + mensaje + "\x1b[0m"
    elif (color == "y"):
        mensajeColor = "\x1b[33m" + mensaje + "\x1b[0m"
    elif (color == "b"):
        mensajeColor = "\x1b[34m" + mensaje + "\x1b[0m"
    elif (color == "p"):
        mensajeColor = "\x1b[35m" + mensaje + "\x1b[0m"
    elif (color == "c"):
        mensajeColor = "\x1b[36m" + mensaje + "\x1b[0m"
    else:
        mensajeColor = mensaje
    mensaje_out = " " + mensajeColor
    print(mensaje_out)


#########################################################################
def fuaa_assert(condicion,
                mensajeFalse="El resultado no es válido.",
                mensajeTrue="Resultado validado."):
    """
    Función similar al assert.
    """
    # Custom assert.
    if (condicion):
        printcolor(mensajeTrue, "g")
    else:
        printcolor(mensajeFalse, "r")


#########################################################################
def son_iguales(x1, x2, threshold=_THRESHOLD):
    """
    Evaluar si dos elementos son iguales o no, con una tolerancia dada (threshold).
    """
    if isinstance(x1, float) or isinstance(x1, int):
        dif = np.abs(x1 - x2)
        condicion = (dif < threshold)
    elif isinstance(x1, np.ndarray):
        dif = np.sqrt(np.sum((x1 - x2)**2)) / x1.size
        condicion = (dif < threshold)
    else:
        printcolor("Ningún tipo validado para son_iguales()", "r")
        condicion = False
    return condicion


#########################################################################
def validar_parametros(parametros, min_params):
    """
    Validar número de parámetros.
    """
    condicion = (len(parametros) >= min_params)
    if not condicion:
        printcolor("[validar_resultado] Insuficientes parámetros (\"%s\"), se necesitan %d, hay %d." %
                   (parametros[0], min_params, len(parametros)), "r")
    return condicion

#########################################################################

